@@ -1,7 +1,7 @@
 //! Functions dealing with obtaining and referencing singleton databases
 
 use crate::{
-    conn::{new_connection_pool, ConnectionPool, PConn, DATABASE_HANDLES},
+    conn::{new_connection_pool, ConnectionPool, DbSyncLevel, PConn, DATABASE_HANDLES},
     prelude::*,
 };
 use derive_more::Into;
@@ -81,17 +81,29 @@ pub struct DbWrite(DbRead);
 impl DbWrite {
     /// Create or open an existing database reference,
     pub fn open(path_prefix: &Path, kind: DbKind) -> DatabaseResult<DbWrite> {
+        Self::open_with_sync_level(path_prefix, kind, DbSyncLevel::default())
+    }
+
+    pub fn open_with_sync_level(
+        path_prefix: &Path,
+        kind: DbKind,
+        sync_level: DbSyncLevel,
+    ) -> DatabaseResult<DbWrite> {
         let path = path_prefix.join(kind.filename());
         if let Some(v) = DATABASE_HANDLES.get(&path) {
             Ok(v.clone())
         } else {
-            let db = Self::new(path_prefix, kind)?;
+            let db = Self::new(path_prefix, kind, sync_level)?;
             DATABASE_HANDLES.insert_new(path, db.clone());
             Ok(db)
         }
     }
 
-    pub(crate) fn new(path_prefix: &Path, kind: DbKind) -> DatabaseResult<Self> {
+    pub(crate) fn new(
+        path_prefix: &Path,
+        kind: DbKind,
+        sync_level: DbSyncLevel,
+    ) -> DatabaseResult<Self> {
         let path = path_prefix.join(kind.filename());
         let parent = path
             .parent()
@@ -100,8 +112,10 @@ impl DbWrite {
             std::fs::create_dir_all(parent)
                 .map_err(|_e| DatabaseError::DatabaseMissing(parent.to_owned()))?;
         }
-        let pool = new_connection_pool(&path, kind.clone());
+        let pool = new_connection_pool(&path, kind.clone(), sync_level);
         let mut conn = pool.get()?;
+        // set to faster write-ahead-log mode
+        conn.pragma_update(None, "journal_mode", &"WAL".to_string())?;
         crate::table::initialize_database(&mut conn, &kind)?;
 
         Ok(DbWrite(DbRead {
@@ -134,9 +148,7 @@ impl DbWrite {
         match map.get(kind) {
             Some(s) => s.clone(),
             None => {
-                let num_cpus = num_cpus::get();
-                let num_read_threads = if num_cpus < 4 { 4 } else { num_cpus / 2 };
-                let s = Arc::new(Semaphore::new(num_read_threads));
+                let s = Arc::new(Semaphore::new(num_read_threads()));
                 map.insert(kind.clone(), s.clone());
                 s
             }
@@ -147,7 +159,7 @@ impl DbWrite {
     /// connection pool, useful for testing.
     #[cfg(any(test, feature = "test_utils"))]
     pub fn test(tmpdir: &tempdir::TempDir, kind: DbKind) -> DatabaseResult<Self> {
-        Self::new(tmpdir.path(), kind)
+        Self::new(tmpdir.path(), kind, DbSyncLevel::default())
     }
 
     /// Remove the db and directory
@@ -160,6 +172,12 @@ impl DbWrite {
     }
 }
 
+pub fn num_read_threads() -> usize {
+    let num_cpus = num_cpus::get();
+    let num_threads = num_cpus.checked_div(2).unwrap_or(0);
+    std::cmp::max(num_threads, 4)
+}
+
 /// The various types of database, used to specify the list of databases to initialize
 #[derive(Clone, Debug, PartialEq, Eq, Hash, derive_more::Display)]
 pub enum DbKind {
@@ -218,7 +236,6 @@ pub trait WriteManager<'e> {
     /// transaction, and commit the transaction after the closure has run.
     /// If there is a SQLite error, recover from it and re-run the closure.
     // FIXME: B-01566: implement write failure detection
-    #[cfg(feature = "test_utils")]
     fn with_commit_sync<E, R, F>(&'e mut self, f: F) -> Result<R, E>
     where
         E: From<DatabaseError>,
